#include <LPC11xx.h>                                   // LPC1100 definitions

#include "global.h"

#define FLASH_POINTER        (BYTE *)0x00003000        // sector 3
#define IAP_LOCATION         0x1fff1ff1
#define PREPARE_SECTOR       50
#define COPY_RAM_TO_FLASH    51
#define ERASE_SECTOR         52

typedef void (*IAP)(LONG *, LONG *);

const IAP iap_entry = (IAP)IAP_LOCATION;

const BYTE POWER_ON_LEVEL    __attribute__((at(0x00003000))) = 254;
const BYTE SYS_FAILURE_LEVEL __attribute__((at(0x00003001))) = 254;
const BYTE MIN_LEVEL         __attribute__((at(0x00003002))) = PHY_MIN_LEVEL;
const BYTE MAX_LEVEL         __attribute__((at(0x00003003))) = 254;
const BYTE FADE_RATE         __attribute__((at(0x00003004))) = 7;
//const BYTE FADE_TIME         __attribute__((at(0x00003005))) = 0;
const BYTE FADE_TIME         __attribute__((at(0x00003005))) = 4;
const BYTE RANDOM_ADDRESS_H  __attribute__((at(0x00003006))) = 0xFF;
const BYTE RANDOM_ADDRESS_M  __attribute__((at(0x00003007))) = 0xFF;
const BYTE RANDOM_ADDRESS_L  __attribute__((at(0x00003008))) = 0xFF;
//const BYTE SHORT_ADDRESS     __attribute__((at(0x00003009))) = 0xFF;
const BYTE SHORT_ADDRESS     __attribute__((at(0x00003009))) = 0x11;
const BYTE GROUP_0_7         __attribute__((at(0x0000300A))) = 0;
const BYTE GROUP_8_15        __attribute__((at(0x0000300B))) = 0;
const BYTE SCENE[16]         __attribute__((at(0x0000300C))) = {255,255,255,255,255,255,255,255,
                                                                255,255,255,255,255,255,255,255};

static LONG cmd[5];
static LONG res[4];
static BYTE buf[28];


static void FlashIAP(void)
{
    __disable_irq();

    cmd[0] = PREPARE_SECTOR;                           // prepare sector
    cmd[1] = 3;                                        // start sector 3 - 0x00003000
    cmd[2] = 3;                                        // end sector 3
    iap_entry(cmd, res);

    cmd[0] = ERASE_SECTOR;                             // erase sector
    cmd[1] = 3;                                        // start sector 3
    cmd[2] = 3;                                        // end sector 3
    cmd[3] = SystemFrequency/1000;                     // system clock in KHz
    iap_entry(cmd,res);                                // error checking recommended

    cmd[0] = PREPARE_SECTOR;                           // prepare sector
    cmd[1] = 3;                                        // start sector 3 - 0x00003000
    cmd[2] = 3;                                        // end sector 3
    iap_entry(cmd, res);

    cmd[0] = COPY_RAM_TO_FLASH;                        // copy ram to flash
    cmd[1] = 0x00003000;                               // destination
    cmd[2] = (LONG)buf;                                // source
    cmd[3] = 256;                                      // nr of bytes
    cmd[4] = SystemFrequency/1000;                     // system clock in KHz
    iap_entry(cmd,res);                                // error checking recommended

    __enable_irq();
}

void FlashWrite(BYTE n, BYTE new)
{
  BYTE i;

    for (i = 0; i < 28; i++)                           // Copy data to buffer
        buf[i] = *(FLASH_POINTER + i);

    buf[n] = new;
    FlashIAP();
}

void FlashRandom(LONG new)
{
  BYTE i;

    for (i = 0; i < 28; i++)                           // Copy data to buffer
        buf[i] = *(FLASH_POINTER + i);

    buf[6] = new >> 16;                                // random address H
    buf[7] = new >> 8;                                 // random address M
    buf[8] = new;                                      // random address L
    FlashIAP();
}

void FlashReset(void)
{
  BYTE i;

    for (i = 0; i < 28; i++)                           // Copy data to buffer
        buf[i] = 255;

    buf[0]  = 254;
    buf[1]  = 254;
    buf[2]  = PHY_MIN_LEVEL;
    buf[3]  = 254;
    buf[4]  = 7;
    buf[5]  = 0;
    buf[10] = 0;
    buf[11] = 0;

    FlashIAP();
}
